#!/usr/bin/env python
import os
import sys
import traceback
import codecs
import subprocess

#add lib path
playbook_file = os.path.realpath(sys.argv[0])
playbook_path=os.path.split(playbook_file)[0]   

lib_path=os.path.join(playbook_path,'library')
if os.path.exists(lib_path) is True:
    sys.path.insert(0,lib_path)
    lib_path=os.path.join(lib_path,'securecat')
    if os.path.exists(lib_path) is True:
        sys.path.insert(0,lib_path)
        modulepath = os.environ.get('CSC_MODULE_PATH')
        if modulepath is None:
            os.environ['CSC_MODULE_PATH'] = lib_path
    
from utils import playbook
from linux.nginxparser import load_from_file
from linux.nginxparser import build_directives
from linux.nginxparser import set_interval
from linux.nginxparser import find_all_in_directives


class MyPlaybook(playbook.Playbook):
    
    def set_extra_vars_pre(self):
        self._runcontext.set_extra_var('NGINX_PID', '')
        self._runcontext.set_extra_var('NGINX_CONFIG_FILES','/etc/nginx/nginx.conf')
        self._runcontext.set_extra_var('NGINX_BIN','/usr/sbin/nginx')

    def set_extra_vars_post(self):
        # get NGINX_CONFIG_FILES from variables
        var_nginx_pid = self._runcontext.get_extra_var('PID')
        if var_nginx_pid == '':
            nginx_pid = "netstat -lntup | grep nginx | awk -F' ' '{print $7}' | awk -F'/' '{print $1}' | tail -n 1"
            p = subprocess.Popen(nginx_pid, env=os.environ, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
            self._runcontext.sleep()
            var_nginx_pid = p.stdout.read()
            var_nginx_pid = var_nginx_pid.split("\n")[0]
        self._runcontext.set_extra_var('NGINX_PID', var_nginx_pid)
        #get NGINX_CONFIG_FILES from variables
        var_nginx_cfg_file = self._runcontext.get_extra_var('CONF_PATH')
        if var_nginx_cfg_file is None:
            raise Exception('NGINX_CONFIG_FILES variable not found.')
        else:
            self._runcontext.log_debug('NGINX_CONFIG_FILES = %s' % var_nginx_cfg_file)
        
        main_cfg_file_realpath = os.path.realpath(var_nginx_cfg_file)
        self._runcontext.set_extra_var('NGINX_CONFIG_FILES',main_cfg_file_realpath)
        if not os.path.exists(main_cfg_file_realpath):
            raise Exception('Nginx main config file is not exist.')
        
        #load main config file, parse nginx configure and get all include config files
        base_path = os.path.split(main_cfg_file_realpath)[0]        
        try:
            set_interval(self._runcontext.get_poll_interval())
            ret = load_from_file(main_cfg_file_realpath)
            if ret[0] != 0:
                raise Exception('Parse nginx config file failed.')
            model = build_directives(ret[1], base_path=base_path)
            self._runcontext.sleep()
            includes = find_all_in_directives(model, 'include')
            files = []
            files.append(main_cfg_file_realpath)
            for i in includes:
                for j in i.get_files():
                    if j not in files:
                        files.append(j)
            
            self._runcontext.set_extra_var('ALL_NGINX_CONFIG_FILES',files)
            self._runcontext.set_parameter('_nginx_cfg_model',model)
            
        except:
            self._runcontext.log_debug(traceback.format_exc())
            traceback.print_exc()

    def get_tasks(self, dir):
        ret = []        

        ret.append([dir, 'a_1_1_1.py'])
        ret.append([dir, 'a_1_1_2.py'])
        ret.append([dir, 't_2_1_2.py'])
        ret.append([dir, 't_2_1_3.py'])
        ret.append([dir, 't_2_1_4.py'])
        ret.append([dir, 't_2_1_5.py'])

        ret.append([dir, 't_2_2_1.py'])
        ret.append([dir, 't_2_2_2.py'])
        ret.append([dir, 't_2_2_4.py'])
        ret.append([dir, 't_2_2_5.py'])
        ret.append([dir, 't_2_2_6.py'])
        ret.append([dir, 't_2_2_7.py'])
        ret.append([dir, 't_2_2_9.py'])
        ret.append([dir, 't_2_2_10.py'])

        ret.append([dir, 't_2_3_1.py'])
        ret.append([dir, 't_2_3_2.py'])
        ret.append([dir, 't_2_3_3.py'])
        ret.append([dir, 't_2_3_4.py'])

        ret.append([dir, 't_2_4_1.py'])

        ret.append([dir, 't_2_5_1.py'])

        ret.append([dir, 't_2_6_1.py'])
        ret.append([dir, 't_2_6_2.py'])
        ret.append([dir, 't_2_6_3.py'])
        ret.append([dir, 't_2_6_4.py'])

        ret.append([dir, 't_2_7_1.py'])
        ret.append([dir, 't_2_7_2.py'])
        ret.append([dir, 't_2_7_3.py'])
        ret.append([dir, 't_2_7_4.py'])
        ret.append([dir, 't_2_7_6.py'])
        ret.append([dir, 't_2_7_7.py'])
        ret.append([dir, 't_2_7_8.py'])

        ret.append([dir, 't_2_8_1.py'])

        return ret
    
if __name__ == '__main__':
    
    myplaybook = MyPlaybook()
    myplaybook.set_role('hw')
    myplaybook.run()
    

