﻿# -*- coding: UTF-8 -*-
import traceback

from linux.nginxparser import find_in_directives

def get_tags():
    return ['1','1.1','1.1.2']
    
def run_task(runcontext):
    rc,out=run_audit(runcontext)
    return rc,out
    
def run_audit(runcontext):
    
    ret_code = 0
    ret_msg = ''
    
    curtemplate=runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''
        
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
        
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code,ret_msg
    
    try:
        #get all location nodes
        errMsg = ''
        checkMsg = []
        location_count = 0
        locationblocks = find_in_directives(cfg_model,['http','server','location'])

        if 0 == len(locationblocks):
            errMsg += 'No location directive was found.'
        else:
            for locationblock in locationblocks:
                if 2 == len(locationblock.args) and '~*' == locationblock.args[0]:
                    key = [svalue.key for svalue in locationblock.value]
                    value = [svalue.value for svalue in locationblock.value]
                    if '\.php' in locationblock.args[1]:
                        location_count += 1
                        if 'deny' in key and 'all' in value:
                            if 'failed' in errMsg:
                                errMsg = errMsg.replace('failed','')
                            if r'/((.*)\s+\\0\.php)' not in locationblock.args[1]:
                                errMsg += locationblock.args[1]
                            else:
                                checkMsg.append("location %s {\n  ...  \ndeny  all;\n  ...  \n}\n" % (" ".join(locationblock.args)))
                        else:
                            errMsg += 'deny all not set in %s. ' % locationblock.args
                elif 2 < len(locationblock.args) and '~*' == locationblock.args[0]:
                    errMsg += 'Location setting error in %s.' % locationblock.args
                    if '\.php' in locationblock.args[1]:
                        location_count += 1

        if location_count == 0:
            errMsg = "No location exists for this configuration."

        if 0 == len(errMsg):
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "".join(checkMsg)
        else:
            tmp_vars["checkvalue1"] = errMsg
            ret_code = -1
  
        rc,out = curtemplate.render(runcontext.get_template_file('a_1_1_2.j2'),**tmp_vars)
        
        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out
            
    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code,ret_msg
