﻿# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_in_directives

def get_tags():
    return ['2','2.2','2.2.1']
    
def run_task(runcontext):
    rc,out=run_audit(runcontext)
    return rc,out
    
def run_audit(runcontext):

    curtemplate=runcontext.get_template()
    tmpVars = {}
    tmpVars["policyresult"] = "failed"
    tmpVars["checkresult1"] = "failed"
    tmpVars["checkvalue1"] = ''
        
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
        
    if cfg_model is None:
        retCode = -2
        retMsg = 'Parse Nginx configure file failed.'
        return retCode,retMsg
    
    try:
        #get all http nodes
        locationlocks = find_in_directives(cfg_model,['http','server','location'])
        runcontext.log_debug("locationlocks: %s" % str(locationlocks))
        location = []
        checkflag = False
        limit_count = 0
        deny_count = 0
        
        for locationlock in locationlocks:

            #check server_tokens off;
            curLocationargs = ' '.join(locationlock.args)

            if curLocationargs.strip() == "/":
                checkflag = False
                lvalues = []
                location.append("location %s {"%curLocationargs)
                key_list = [sub.key for sub in locationlock.value]
                if 'limit_except' not in key_list:
                    limit_count -= 1
                for sub in locationlock.value:
                    flag = False
                    key = sub.key
                    if key.strip() == '#':
                        continue
                    if key == 'limit_except':
                        limit_count += 1
                        rargs = sub.args
                        values = sub.value
                        lvalues.append("%s %s {"%(key, ' '.join(rargs)))
                        for method in rargs:
                            if "DELETE" in method or "PUT" in method or "TRACE" in method or "OPTIONS" in method:
                                flag = False
                                break
                            else:
                                flag = True

                        if flag:
                            for ele in values:
                                key = ele.key
                                value = ele.value
                                
                                if key == 'deny':
                                    deny_count += 1
                                    match_found = re.match(r"\s*all",value)
                                    if match_found != None:
                                        checkflag = True
   
                        for ele in values:
                            key = ele.key
                            value = ele.value
                            lvalues.append("%s %s"%(key, value))
                        lvalues.append("}")
                        location.extend(lvalues)

                location.append("}")

        if checkflag and limit_count == deny_count:
            retCode = 0
            tmpVars["policyresult"] = "pass"
            tmpVars["checkresult1"] = "pass"
            tmpVars["checkvalue1"] = "\n".join(location)
        else:
            retCode = -1
            tmpVars["checkvalue1"] = "\n".join(location)
            
        rc,out = curtemplate.render(runcontext.get_template_file('t_2_2_1.j2'),**tmpVars)
        
        if rc == False:
            retCode = -2
            retMsg = ('Failed to render template. %s' % out)
        else:
            retMsg = out
            
    except:
        retCode = -2
        retMsg = traceback.format_exc()

    return retCode,retMsg
