﻿# -*- coding: UTF-8 -*-
import os
import re
import traceback

from linux.nginxparser import find_in_directives
from linux.nginxparser import load_from_file
from linux.nginxparser import build_directives


def get_tags():
    return ['2', '2.2', '2.2.5']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def include(cfg_path, sub):
    base_cfg = os.path.split(cfg_path)[0]
    value = sub.value
    if os.path.isabs(value):
        path = value
    else:
        path = os.path.join(base_cfg, value)
    base_path = os.path.split(path)[0]
    ret = load_from_file(path)
    if ret[0] != 0:
        raise Exception('Parse nginx config file failed.')
    return build_directives(ret[1], base_path=base_path)


def location_block(locationcount, vs_add_header, sub1, locationvalues, scan_failed_locationcount, serverErrMsg, location_num, httpErrMsg):
    locationcount = locationcount + 1
    vl_add_header = vs_add_header
    locationvalue = ' '.join(sub1.args)
    locationErrMsg = []
    locationErrMsg.append('        location %s {' % locationvalue)

    # check in location
    for sub2 in sub1.value:
        key2 = sub2.key
        if 'port_in_redirect' == key2:
            vl_add_header = sub2.value.lower()
            locationErrMsg.append('            port_in_redirect %s;' % vl_add_header)
    locationErrMsg.append('        }')

    if None == re.match(r'off', vl_add_header):
        locationvalues.append(locationvalue)
        scan_failed_locationcount += 1
        # 防止Location太多，导致返回的配置值超大，只记录部分不满足项即可
        if scan_failed_locationcount < 3:
            serverErrMsg.extend(locationErrMsg)
            httpErrMsg.extend(serverErrMsg)
        elif scan_failed_locationcount == 3:
            serverErrMsg.extend(['      ......'])
            httpErrMsg.extend(serverErrMsg)

    return locationcount, locationvalues


def server_block(servercount, vh_add_header, sub, locationvalues, httpErrMsg, servernames, cfg_path):
    servercount = servercount + 1
    locationcount = 0
    location_num = 0
    scan_failed_locationcount = 0
    servername = ''
    vs_add_header = vh_add_header
    serverErrMsg = []
    serverErrMsg.append('    server {')
    # check in server
    key1_list = [sub1.key for sub1 in sub.value]
    for key1 in key1_list:
        if key1 == 'location':
            location_num += 1
        if 'port_in_redirect' in key1_list:
            for sub1 in sub.value:
                key1 = sub1.key
                if key1 == 'port_in_redirect':
                    vs_add_header = sub1.value.lower()
    for sub1 in sub.value:
        key1 = sub1.key
        if 'port_in_redirect' == key1:
            vs_add_header = sub1.value.lower()
            serverErrMsg.append('        port_in_redirect %s;' % vs_add_header)
        elif 'server_name' == key1:
            servername = sub1.value
            serverErrMsg.append('        server_name %s;' % servername)
        elif key1 == 'include':
            model = include(cfg_path, sub1)
            port_in_redirect = find_in_directives(model, ['port_in_redirect'])
            locationblocks = find_in_directives(model, ['location'])
            if len(port_in_redirect) != 0:
                if port_in_redirect[0].key == 'port_in_redirect':
                    vs_add_header = port_in_redirect[-1].value.lower()
                    serverErrMsg.append('        port_in_redirect %s;' % vs_add_header)
            for locationblock in locationblocks:
                # check in location
                locationcount, locationvalues = location_block(locationcount, vs_add_header, locationblock, locationvalues, scan_failed_locationcount, serverErrMsg, location_num, httpErrMsg)
        elif 'location' == key1:
            # check in location
            locationcount, locationvalues = location_block(locationcount, vs_add_header, sub1, locationvalues, scan_failed_locationcount, serverErrMsg, location_num, httpErrMsg)

    if 0 == locationcount:
        if None == re.match(r'off', vs_add_header):
            servernames.append(servername)
            serverErrMsg.append('            port_in_redirect %s;' % vs_add_header)
    serverErrMsg.append('    }')

    return servercount, locationcount, servername, serverErrMsg, httpErrMsg


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
    cfg_path = runcontext.get_extra_var('NGINX_CONFIG_FILES')

    # Syntax: port_in_redirect on | off;
    # Default: port_in_redirect on;
    # Context: http, server, location
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all http nodes
        httpblocks = find_in_directives(cfg_model, ['http'])
        runcontext.log_debug("httpblocks: %s" % str(httpblocks))
        httpindexs = []
        servernames = []
        locationvalues = []
        servercount = 0
        locationcount = 0
        scan_failed_locationcount = 0
        httpindex = 1
        totalErrMsg = []
        totalCheckMsg = []

        for httpblock in httpblocks:
            servercount = 0
            vh_add_header = 'on'
            vs_add_header = ''
            vl_add_header = ''
            httpErrMsg = []
            httpErrMsg.append('http {')

            # check in http
            key_list = [sub.key for sub in httpblock.value]
            if 'port_in_redirect' in key_list:
                for sub in httpblock.value:
                    key = sub.key
                    if key == 'port_in_redirect':
                        vh_add_header = sub.value.lower()
            for sub in httpblock.value:
                key = sub.key
                if isinstance(key, str):
                    if 'port_in_redirect' == key:
                        vh_add_header = sub.value.lower()
                        httpErrMsg.append('    port_in_redirect %s;' % vh_add_header)
                    elif key == 'include':
                        model = include(cfg_path, sub)
                        port_in_redirect = find_in_directives(model, ['port_in_redirect'])
                        serverblocks = find_in_directives(model, ['server'])
                        if len(port_in_redirect) != 0:
                            if port_in_redirect[0].key == 'port_in_redirect':
                                vh_add_header = port_in_redirect[-1].value.lower()
                                httpErrMsg.append('    port_in_redirect %s;' % vh_add_header)
                        for serverblock in serverblocks:
                            # check in server
                            servercount, locationcount, servername, serverErrMsg, httpErrMsg = server_block(servercount, vh_add_header, serverblock, locationvalues, httpErrMsg, servernames, cfg_path)
                elif isinstance(key, list):
                    key = key[0]
                    if key == 'server':
                        # check in server
                        servercount, locationcount, servername, serverErrMsg, httpErrMsg = server_block(servercount, vh_add_header, sub, locationvalues, httpErrMsg, servernames, cfg_path)
                else:
                    pass

            if 0 == servercount:
                if None == re.match(r'off', vh_add_header):
                    httpindexs.append(httpindex)
                    httpErrMsg.append('    port_in_redirect %s;' % vh_add_header)

            httpindex = httpindex + 1
            httpErrMsg.append('}')
            totalErrMsg.extend(httpErrMsg)
            totalCheckMsg.append('http {\n   ...\n   port_in_redirect off;\n   ...\n   Server {\n   ...\n}\n...\n}')

        if len(locationvalues) > 0:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(totalErrMsg)
        elif len(servernames) > 0 and locationcount == 0:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(totalErrMsg)
        elif len(httpindexs) > 0 and locationcount == 0 and servercount == 0:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(totalErrMsg)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(totalCheckMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_2_5.j2'), **tmp_vars)
        if rc == False:
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
