﻿# -*- coding: UTF-8 -*-
import traceback

from linux.nginxparser import find_in_directives


def get_tags():
    return ['2', '2.2', '2.2.7']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all http nodes
        locationblocks = find_in_directives(cfg_model, ['http', 'server', 'location'])
        runcontext.log_debug("locationblocks: %s" % str(locationblocks))
        checkresult = True
        isExists = False  # exists allow all flag
        checkflag = False
        errMsg = []
        checkresultList = []
        tmpMsg = ''
        vlocations = []
        rlocations = []
        lvalues = []
        rvalues = []

        if 0 == len(locationblocks):
            checkresult = False
            errMsg.append('location node is not configured,')
            ret_code = -2
            vlocations.append("location \n{ \n}")
            rlocations.append("location \n{ \n}")
        else:
            failed_location_count = 0
            # check in every location
            for locationblock in locationblocks:
                if isExists:
                    if tmpMsg not in errMsg:
                        errMsg.append(tmpMsg)
                    continue
                checkflag = False
                tmpMsg = ''

                if failed_location_count < 3:
                    vlocations.append("location %s {" % (" ".join(locationblock.args)))
                rlocations.append("location %s {" % (" ".join(locationblock.args)))

                lvalues = []
                rvalues = []
                # check in single location
                for sub in locationblock.value:
                    key = sub.key
                    values = sub.value
                    if key.strip() == '#':
                        continue
                    if isinstance(values, str):
                        lvalues.append("    %s %s" % (key, values))
                        rvalues.append("    %s %s" % (key, values))
                    if isinstance(values, list):
                        key = values[0].key
                        values = values[0].value
                        lvalues.append("    %s %s %s" % (sub.key, key, values))
                        rvalues.append("    %s %s %s" % (sub.key, key, values))
                    if key == "allow" and values == "all":
                        checkflag = False
                        isExists = True
                        tmpMsg = 'allow all is configured in node %s %s,' % (locationblock.key, locationblock.args)
                        break
                    elif key == 'deny':
                        if values == "all":
                            checkflag = True
                            lvalues = []
                        else:
                            tmpMsg = 'deny all is not configured in node %s %s,' % (
                            locationblock.key, locationblock.args)
                if checkflag == False:
                    if 0 == len(tmpMsg):  # there is no deny directive configured in location
                        tmpMsg = 'There is no deny directive configured in node %s %s,' % (
                        locationblock.key, locationblock.args)
                        if tmpMsg not in errMsg:
                            errMsg.append(tmpMsg)
                    else:
                        if tmpMsg not in errMsg:
                            errMsg.append(tmpMsg)
                    checkresultList.append('False')
                    lvalues.append("}")
                    failed_location_count += 1

                    if failed_location_count < 4:
                        vlocations.extend(lvalues)
                    elif failed_location_count == 4:
                        vlocations.append('......')
                    continue
                else:
                    if failed_location_count < 3:
                        vlocations.append("...")
                        vlocations.append("deny all")
                        vlocations.append("}")

                    checkresultList.append('True')
                    rvalues.append("}")
                    rlocations.extend(rvalues)
            if 'False' in checkresultList:
                checkresult = False

        if checkresult == False:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(vlocations)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(rlocations)

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_2_7.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
