﻿# -*- coding: UTF-8 -*-
import os
import re
import json

from linux.nginxparser import find_all_in_directives

def get_tags():
    return ['2','2.3','2.3.2']
  
def run_task(runcontext):
    ansibleParams=runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansibleParams,"chdir",None)
    runcontext.set_mod_params_instance(ansibleParams,"executable",None)

    module=runcontext.get_module_file("shell.py")
    rc,out=run_audit(runcontext,ansibleParams,module)
    return rc,out
    
def run_audit(runcontext,ansibleParams,module):
    
    retCode = 0
    retMsg = ''
    
    curtemplate=runcontext.get_template()
    allcfgs = runcontext.get_extra_var('ALL_NGINX_CONFIG_FILES')
    nginx_pid = runcontext.get_extra_var('NGINX_PID')
    nginx_pid = runcontext.replace_inject_characters(nginx_pid)
    tmpVars = {}
    tmpVars["policyresult"] = "failed"
    tmpVars["checkvalue1"] = ''

    nginx_user = '''ps -uwp %s | grep nginx | awk -F' ' '{print $1}' ''' % nginx_pid
    runcontext.set_mod_params_instance(ansibleParams, "command", nginx_user)
    arc3, astdout3, astderr3 = runcontext.invoke_module(module, ansibleParams)
    runcontext.sleep()
    if arc3 != 0:
        return -2, "Failed to invoke module: %s." % astdout3
    resultjson3 = json.loads(astdout3)
    astdout3 = resultjson3["stdout"]
    runcontext.log_debug_json(resultjson3)
    match3_found = re.match("^\s*root\s*$", astdout3)

    nginx_group = '''GID=$(cat /etc/passwd | grep "^\s*%s:" | awk -F':' '{print $4}');cat /etc/group | grep ":$GID:" | awk -F':' '{print $1}' ''' % astdout3
    runcontext.set_mod_params_instance(ansibleParams, "command", nginx_group)
    arc4, astdout4, astderr4 = runcontext.invoke_module(module, ansibleParams)
    runcontext.sleep()
    if arc4 != 0:
        return -2, "Failed to invoke module: %s." % astdout4
    resultjson4 = json.loads(astdout4)
    astdout4 = resultjson4["stdout"]
    runcontext.log_debug_json(resultjson4)

    if match3_found:
        cfg_model = runcontext.get_parameter('_nginx_cfg_model')
        tmpusers = find_all_in_directives(cfg_model, 'user')
        runcontext.log_debug("tmpusers: %s" % str(tmpusers))
        if len(tmpusers) == 0:
            runuser = 'nobody'
            rungroup = 'nobody'
        else:
            values = runcontext.replace_inject_characters(tmpusers[-1].value)
            values_list = values.split()
            if len(values_list) == 1:
                runuser = values
                rungroup = values
            else:
                runuser = values_list[0]
                rungroup = values_list[-1]
    else:
        runuser = astdout3
        rungroup = astdout4
        
    for bcfg in allcfgs:
        
        dir = os.path.split(bcfg)[0]
        fileName = os.path.split(bcfg)[1]
        tmpVars["checkresult1"] = "failed"

        afile = runcontext.replace_inject_characters(os.path.join(dir,fileName))
        tmpVars["scancommand1"] = "access=$(stat -L --format=\"%s\" %s);user=$(stat -L --format=\"%s\" %s);group=$(stat -L --format=\"%s\" %s);if (( (0$access & 0177) != 0 ))  ||  [ \"$user\" != 'root' -a \"$user\" != '%s' ]  ||  [ \"$group\" != 'root' -a \"$group\" != '%s' ];then echo \"%s $user:$group $access\";else echo \"%s $user:$group $access\";fi" % ("%a",afile,"%U",afile,"%G",afile,runuser,rungroup,afile,afile)
        runcontext.set_mod_params_instance(ansibleParams,"command",tmpVars["scancommand1"])
        arc1,astdout1,astderr1=runcontext.invoke_module(module,ansibleParams)
        runcontext.sleep()

        if arc1 != 0:
            return (-2, "Failed to invoke module: %s."%astdout1)

        resultjson1 = json.loads(astdout1)
        astdout1 = resultjson1["stdout"]
        astderr1 = resultjson1["stderr"]
        runcontext.log_debug_json(resultjson1)
        
        match1_found=re.match(r"^.+\s+(%s:%s|root:root)\s*[0-6]00$" % (runuser,rungroup),astdout1)
        if match1_found == None:
            retCode = -1
            tmpVars["checkvalue1"] = astdout1
            break
        else:
            retCode = 0
            tmpVars["policyresult"] = "pass"
            tmpVars["checkresult1"] = "pass"
            tmpVars["checkvalue1"] = astdout1
        
    rc,out = curtemplate.render(runcontext.get_template_file('t_2_3_2.j2'),**tmpVars)
    
    if rc == False:
        retCode = -2
        retMsg = ('Failed to render template. %s' % out)
    else:
        retMsg = out

    return retCode,retMsg
