# -*- coding: UTF-8 -*-
import os
import re
import json
import traceback

from linux.nginxparser import find_all_in_directives

def get_tags():
    return ['2', '2.3', '2.3.3']
    
def run_task(runcontext):
    ansibleParams = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansibleParams, "chdir", None)
    runcontext.set_mod_params_instance(ansibleParams, "executable", None)
    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansibleParams, module)
    return rc, out

    
def run_audit(runcontext, ansibleParams, module):
    nginx_pid = runcontext.get_extra_var('NGINX_PID')
    nginx_pid = runcontext.replace_inject_characters(nginx_pid)
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    nginx_user = '''ps -uwp %s | grep nginx | awk -F' ' '{print $1}' ''' % nginx_pid
    runcontext.set_mod_params_instance(ansibleParams, "command", nginx_user)
    arc3, astdout3, astderr3 = runcontext.invoke_module(module, ansibleParams)
    runcontext.sleep()
    if arc3 != 0:
        return -2, "Failed to invoke module: %s." % astdout3
    resultjson3 = json.loads(astdout3)
    astdout3 = resultjson3["stdout"]
    runcontext.log_debug_json(resultjson3)
    match3_found = re.match("^\s*root\s*$", astdout3)

    nginx_group = '''GID=$(cat /etc/passwd | grep "^\s*%s:" | awk -F':' '{print $4}');cat /etc/group | grep ":$GID:" | awk -F':' '{print $1}' ''' % astdout3
    runcontext.set_mod_params_instance(ansibleParams, "command", nginx_group)
    arc4, astdout4, astderr4 = runcontext.invoke_module(module, ansibleParams)
    runcontext.sleep()
    if arc4 != 0:
        return -2, "Failed to invoke module: %s." % astdout4
    resultjson4 = json.loads(astdout4)
    astdout4 = resultjson4["stdout"]
    runcontext.log_debug_json(resultjson4)

    if match3_found:
        tmpusers = find_all_in_directives(cfg_model, 'user')
        runcontext.log_debug("tmpusers: %s" % str(tmpusers))
        if len(tmpusers) == 0:
            runuser = 'nobody'
            rungroup = 'nobody'
        else:
            values = runcontext.replace_inject_characters(tmpusers[-1].value)
            values_list = values.split()
            if len(values_list) == 1:
                runuser = values
                rungroup = values
            else:
                runuser = values_list[0]
                rungroup = values_list[-1]
    else:
        runuser = astdout3
        rungroup = astdout4

    retCode = 0
    retMsg = ''
    
    curtemplate = runcontext.get_template()
    tmpVars = {}
    tmpVars["policyresult"] = "failed"
    tmpVars["checkresult1"] = "failed"
    tmpVars["checkvalue1"] = ''
    tmpVars["checkresult2"] = "failed"
    tmpVars["checkvalue2"] = ''
        
    cfgPath = runcontext.get_extra_var('NGINX_CONFIG_FILES')

    base_cfg = os.path.split(cfgPath)[0]
    m = re.search("conf",base_cfg)
    if m != None:
        base_cfg = base_cfg[0:m.start()].strip().rstrip(os.sep)
    
    if cfg_model is None:
        retCode = -2
        retMsg = 'Parse Nginx configure file failed.'
        return retCode, retMsg
    
    try:
        # get all http nodes
        errloglocks = find_all_in_directives(cfg_model, 'error_log')
        acsloglocks = find_all_in_directives(cfg_model, 'access_log')        
        errmsg1 = []
        cmds1 = []
        if len(errloglocks) == 0:
            errmsg1.append("error_log not configured")
        #error_log文件权限检查
        #语法: error_log file   stderr   syslog:server=address[,parameter=value] [debug   info   notice   warn   error   crit   alert   emerg];
        #默认值: error_log logs/error.log error;
        #配置段: main, http, server, location
        for block in errloglocks:
            runcontext.log_debug(block)
            value = block.value.strip()
            values = value.split()
            log_file = ''
            if len(values) == 0:
                errmsg1.append('error_log configure value is incorrect : %s.' % value)
                continue
            #stderr
            if values[0] == 'stderr':
                errmsg1.append('error_log stderr;')
                continue
            
            if values[0] == '/dev/null':
                errmsg1.append('error_log /dev/null;')
                continue            
            
            if values[0].startswith('syslog'):
                continue
            
            log_file = values[0]
            if "$" in log_file:
                errmsg1.append("Change %s to the real path" % log_file)
                break
            if os.path.isabs(log_file) is False:
                #相对路径，一般为标准部署
                log_file = os.path.join(base_cfg,log_file)
            if os.path.exists(log_file) is False:
                # 判断路径是否存在
                errmsg1.append("%s does not exist" % log_file)
                break
            
            thiscmd = "find %s " % runcontext.replace_inject_characters(log_file)
            thiscmd = thiscmd + r'''-exec stat -L --print "%n %U:%G %a;" {} \; 2>/dev/null'''
            cmds1.append(thiscmd)
            runcontext.set_mod_params_instance(ansibleParams, "command", thiscmd)
            arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansibleParams)
            runcontext.log_debug_invoke_result(arc1,astdout1,astderr1)
            runcontext.sleep()
            if arc1 != 0:
                return (-2, "Failed to invoke module: %s." % astdout1)

            resultjson1 = json.loads(astdout1)
            astdout1 = resultjson1["stdout"]
            astderr1 = resultjson1["stderr"]
            
            match1_found = re.match(r"^.+\s+%s:%s\s*[0-6]00;$" % (runuser,rungroup), astdout1)
            if match1_found == None:
                errmsg1.append("error_log: %s" % astdout1)
            else:
                tmpVars["checkvalue1"] = astdout1

        # access_log文件权限检查
        #语法: access_log path [format [buffer=size [flush=time]]];
        #access_log path format gzip[=level] [buffer=size] [flush=time];
        #access_log syslog:server=address[,parameter=value] [format];
        #access_log off;
        #默认值: access_log logs/access.log combined;
        #配置段: http, server, location, if in location, limit_except
        errmsg2 = []
        cmds2 = []
        if len(acsloglocks) == 0:
            errmsg2.append("access_log not configured")
        for block in acsloglocks:
            runcontext.log_debug(block)
            value = block.value.strip()
            values = value.split()
            log_file = ''
            
            if len(values) == 0:
                errmsg2.append('access_log configure value is incorrect : %s.' % value)
                continue
                
            if values[0] == '/dev/null':
                errmsg2.append('access_log /dev/null;')
                continue
            
            if values[0].startswith('syslog'):
                continue
                
            if values[0] == 'off':
                # errmsg2.append('access_log off;')
                continue

            log_file = values[0]
            if "$" in log_file:
                errmsg2.append("Change %s to the real path" % log_file)
                break
            if os.path.isabs(log_file) is False:
                # 相对路径，一般为标准部署
                log_file = os.path.join(base_cfg,log_file)
            if os.path.exists(log_file) is False:
                # 判断路径是否存在
                errmsg2.append("%s does not exist" % log_file)
                break

            thiscmd = "find %s " % runcontext.replace_inject_characters(log_file)
            thiscmd = thiscmd + r'''-exec stat -L --print "%n %U:%G %a;" {} \; 2>/dev/null'''
            cmds2.append(thiscmd) 
            runcontext.set_mod_params_instance(ansibleParams, "command", thiscmd)
            arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansibleParams)
            runcontext.log_debug_invoke_result(arc1,astdout1,astderr1)
            runcontext.sleep()
            if arc1 != 0:
                return (-2, "Failed to invoke module: %s." % astdout1)

            resultjson1 = json.loads(astdout1)
            astdout1 = resultjson1["stdout"]
            astderr1 = resultjson1["stderr"]
            
            match1_found = re.match(r"^.+\s+%s:%s\s*[0-6]00;$" % (runuser,rungroup), astdout1)
            if match1_found == None:
                errmsg2.append("access_log: %s" % astdout1)
            else:
                tmpVars["checkvalue2"] = astdout1
        
        tmpVars["scancommand1"] = ';'.join(cmds1)
        if len(errmsg1) >0 :
            tmpVars["checkvalue1"] = '\\n'.join(errmsg1)
        else:
            tmpVars["checkresult1"] = "pass"
            
        tmpVars["scancommand2"] = ';'.join(cmds2)
        if len(errmsg2) >0 :
            tmpVars["checkvalue2"] = '\\n'.join(errmsg2)
        else:
            tmpVars["checkresult2"] = "pass"
            
        if tmpVars["checkresult1"] == 'pass' and tmpVars["checkresult2"] == 'pass':
            retCode = 0
            tmpVars["policyresult"] = "pass"
        else:
            retCode = -1            
        
        rc, out = curtemplate.render(runcontext.get_template_file('t_2_3_3.j2'), **tmpVars)
        
        if rc == False:
            retCode = -2
            retMsg = ('Failed to render template. %s' % out)
        else:
            retMsg = out
            
    except:
        retCode = -2
        retMsg = traceback.format_exc()

    return retCode, retMsg
