﻿# -*- coding: UTF-8 -*-
import os
import re
import traceback

from linux.nginxparser import find_in_directives
from linux.nginxparser import load_from_file
from linux.nginxparser import build_directives


def get_tags():
    return ['2', '2.6', '2.6.1']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def include(cfg_path, sub):
    base_cfg = os.path.split(cfg_path)[0]
    value = sub.value
    if os.path.isabs(value):
        path = value
    else:
        path = os.path.join(base_cfg, value)
    base_path = os.path.split(path)[0]
    ret = load_from_file(path)
    if ret[0] != 0:
        raise Exception('Parse nginx config file failed.')
    return build_directives(ret[1], base_path=base_path)


def location_block(locationcount, dv_server, sub1, locationvalues, scan_failed_locationcount, serverErrMsg, location_num, httpErrMsg):
    locationcount = locationcount + 1
    v_location = dv_server
    locationvalue = ' '.join(sub1.args)
    locationErrMsg = []
    locationErrMsg.append('        location %s {' % locationvalue)

    for sub2 in sub1.value:
        key2 = sub2.key
        if key2 == 'add_header' and 'x-xss-protection' in sub2.value.lower():
            v_location = sub2.value.lower()
            locationErrMsg.append('            add_header %s;' % v_location)
    locationErrMsg.append('        }')

    if re.match(r".*1;\s*mode=block", v_location.strip("\"")) == None:
        locationvalues.append(locationvalue)
        scan_failed_locationcount += 1
        # 防止Location太多，导致返回的配置值超大，只记录部分不满足项即可
        if scan_failed_locationcount < 3:
            serverErrMsg.extend(locationErrMsg)
            httpErrMsg.extend(serverErrMsg)
        elif scan_failed_locationcount == 3:
            serverErrMsg.extend(['      ......'])
            httpErrMsg.extend(serverErrMsg)

    return locationcount, locationvalues


def server_block(servercount, dv_http, sub, locationvalues, httpErrMsg, servernames, cfg_path):
    servercount = servercount + 1
    locationcount = 0
    location_num = 0
    scan_failed_locationcount = 0
    dv_server = dv_http
    servername = ''
    serverErrMsg = []
    serverErrMsg.append('    server {')

    # check in server
    key1_list = [sub1.key for sub1 in sub.value]
    for key1 in key1_list:
        if key1 == 'location':
            location_num += 1
        if 'add_header' in key1_list:
            for sub1 in sub.value:
                key1 = sub1.key
                if key1 == 'add_header' and 'x-xss-protection' in sub1.value.lower():
                    dv_server = sub1.value.lower()
    for sub1 in sub.value:
        key1 = sub1.key
        if key1 == 'add_header' and 'x-xss-protection' in sub1.value.lower():
            dv_server = sub1.value.lower()
            serverErrMsg.append('        add_header %s;' % dv_server)
        elif key1 == 'server_name':
            servername = sub1.value
            serverErrMsg.append('        server_name %s;' % servername)
        elif key1 == 'include':
            model = include(cfg_path, sub1)
            add_headers = find_in_directives(model, ['add_header'])
            locationblocks = find_in_directives(model, ['location'])
            if len(add_headers) != 0:
                for add_header in add_headers:
                    if add_header.key == 'add_header' and 'x-xss-protection' in add_header.value.lower():
                        dv_server = add_header.value.lower()
                        serverErrMsg.append('        add_header %s;' % dv_server)
            for locationblock in locationblocks:
                # check in location
                locationcount, locationvalues = location_block(locationcount, dv_server, locationblock, locationvalues, scan_failed_locationcount, serverErrMsg, location_num, httpErrMsg)
        elif key1 == 'location':
            # check in location
            locationcount, locationvalues = location_block(locationcount, dv_server, sub1, locationvalues, scan_failed_locationcount, serverErrMsg, location_num, httpErrMsg)

    if locationcount == 0:
        if re.match(r".*1;\s*mode=block", dv_server.strip("\"")) == None:
            servernames.append(servername)
            serverErrMsg.append('    add_header %s;' % dv_server)
    serverErrMsg.append('    }')

    return servercount, locationcount, servername, serverErrMsg, httpErrMsg


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
    cfg_path = runcontext.get_extra_var('NGINX_CONFIG_FILES')
    totalErrMsg = []
    totalCheckMsg = []
    # Syntax:  add_header name value [always];
    # Default:  —
    # Context:  http, server, location, if in location
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all http nodes
        httpblocks = find_in_directives(cfg_model, ['http'])
        runcontext.log_debug("httpblocks: %s" % str(httpblocks))
        httpindexs = []
        servernames = []
        servercount = 0
        locationvalues = []
        locationcount = 0
        scan_failed_locationcount = 0
        httpindex = 1
        dv_http = ''

        # add_header X-XSS-Protection "1; mode=block";
        for httpblock in httpblocks:
            servercount = 0
            httpErrMsg = []
            httpErrMsg.append('http {')
            # check in http
            key_list = [sub.key for sub in httpblock.value]
            if 'add_header' in key_list:
                for sub in httpblock.value:
                    key = sub.key
                    if key == 'add_header' and 'x-xss-protection' in sub.value.lower():
                        dv_http = sub.value.lower()
            for sub in httpblock.value:
                key = sub.key
                value = sub.value
                if isinstance(key, str):
                    if key == 'add_header' and 'x-xss-protection' in value.lower():
                        dv_http = value.lower()
                        httpErrMsg.append('    add_header %s;' % dv_http)
                    elif key == 'include':
                        model = include(cfg_path, sub)
                        add_headers = find_in_directives(model, ['add_header'])
                        serverblocks = find_in_directives(model, ['server'])
                        if len(add_headers) != 0:
                            for add_header in add_headers:
                                if add_header.key == 'add_header' and 'x-xss-protection' in add_header.value.lower():
                                    dv_http = add_header.value.lower()
                                    httpErrMsg.append('    add_header %s;' % dv_http)
                        for serverblock in serverblocks:
                            # check in server
                            servercount, locationcount, servername, serverErrMsg, httpErrMsg = server_block(servercount, dv_http, serverblock, locationvalues, httpErrMsg, servernames, cfg_path)
                elif isinstance(key, list):
                    key = key[0]
                    if key == 'server':
                        # check in server
                        servercount, locationcount, servername, serverErrMsg, httpErrMsg = server_block(servercount, dv_http, sub, locationvalues, httpErrMsg, servernames, cfg_path)

            if servercount == 0:
                if re.match(r".*1;\s*mode=block", dv_http.strip("\"")) == None:
                    httpindexs.append(httpindex)
                    httpErrMsg.append('    add_header %s;' % dv_http)

            httpindex = httpindex + 1
            httpErrMsg.append('}')
            totalErrMsg.extend(httpErrMsg)
            totalCheckMsg.append("http {\n    …\nadd_header X-XSS-Protection \"1; mode=block\";\n    …\n}")

        runcontext.log_debug(httpindexs)
        runcontext.log_debug(servernames)
        runcontext.log_debug(locationvalues)

        if len(locationvalues) > 0:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(totalErrMsg)
        elif len(servernames) > 0 and locationcount == 0:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(totalErrMsg)
        elif len(httpindexs) > 0 and locationcount == 0 and servercount == 0:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(totalErrMsg)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(totalCheckMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_6_1.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
