# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import (find_in_directives,BlockDirective)

def get_tags():
    return ['2','2.6','2.6.4']
    
def run_task(runcontext):
    rc,out=run_audit(runcontext)
    return rc,out
    
def getList(sub, key):

    key = sub.key
    args = " ".join(sub.args).strip()
    contens = []
    contens.append("%s %s {"%(key,args))
    for lt in sub.value:
        if lt.key.strip() == "#":
            continue
        contens.append("%s %s"%(lt.key, lt.value))
    return contens
    
    
def run_audit(runcontext):
    
    ret_code = 0
    ret_msg = ''
    
    curtemplate=runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''
    vlocations = []
    rlocations = []
    lvalues = []
    rvalues = []
        
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
        
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code,ret_msg
    
    try:
        #get all location nodes
        locationblocks = find_in_directives(cfg_model,['http','server','location'])
        runcontext.log_debug("locationblocks: %s" % str(locationblocks))
        
        failed_location_count = 0
        checkresult = False
        i = 0       #block计数
        j = 0       #当前block中匹配则计数加1
        for locationblock in locationblocks:
            checkflag = False
            i = i + 1
            match_found1 = None
            match_found2 = None
            if failed_location_count < 3:
                vlocations.append("location %s {"%" ".join(locationblock.args))
            rlocations.append("location %s {"%" ".join(locationblock.args))
            lvalues = []
            rvalues = []
            #check if location node configured valid_referers and invalid_referer;
            for sub in locationblock.value:
                key = sub.key
                _args = " ".join(sub.args)
                if key.strip() == '#':
                    continue
                contents = []
                contents.append("%s %s"%(key, sub.value))
                
                if isinstance(sub, BlockDirective):
                    contents = getList(sub,key)
                
                contents = ['    %s' % x for x in contents] if contents else []
                lvalues.extend(contents)

                if key == 'valid_referers':
                    rvalues.extend(contents)
                    value1 = sub.value
                    match_found1 = re.match(r'\s*none\s+blocked',value1)
                elif key == 'if':
                    rvalues.extend(contents)
                    match_found2 = re.match(r'.*\(\s*\$invalid_referer\s*\)',_args)
                else:
                    rvalues.append("...")
                
            if None != match_found1 and None != match_found2:
                checkflag = True
                lvalues = []
                if failed_location_count < 3:
                    vlocations.pop()
                rvalues.append("}")
                rlocations.extend(rvalues)
            else:
                lvalues.append("}")
                if failed_location_count < 3:
                    vlocations.extend(lvalues)
                elif failed_location_count == 3:
                    vlocations.append('......')
                failed_location_count += 1

            if checkflag == True:
                checkresult = True
                j = j + 1

        if i == j and len(locationblocks) != 0:
            checkresult = True
        else:
            checkresult = False

        if checkresult == False:
            ret_code = -1
            tmp_vars["checkvalue1"] = "\n".join(vlocations)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "\n".join(rlocations)
            
        rc,out = curtemplate.render(runcontext.get_template_file('t_2_6_4.j2'),**tmp_vars)
        
        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out
            
    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code,ret_msg
