﻿# -*- coding: UTF-8 -*-
import os
import traceback
import stat
import grp
import pwd

from linux.nginxparser import find_in_directives, find_all_in_directives


def get_tags():
    return ['2','2.7','2.7.1']
    
def run_task(runcontext):
    rc,out=run_audit(runcontext)
    return rc,out

def get_user_name(uid):
    users = pwd.getpwall()
    for user in users:
        if user.pw_uid == uid:
            return user.pw_name
    return None

def get_group_name(gid):
    groups = grp.getgrall()
    for group in groups:
        if group.gr_gid == gid:
            return group.gr_name
    return None
    
def run_audit(runcontext):

    runuser = 'nobody'
    rungroup = 'nobody'
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
    tmpusers = find_all_in_directives(cfg_model,'user')
    runcontext.log_debug("tmpusers: %s" % str(tmpusers))
    if 0 != len(tmpusers):
        runuser = runcontext.replace_inject_characters(tmpusers[-1].value)
        rungroup = runcontext.replace_inject_characters(tmpusers[-1].value)

    ret_code = 0
    ret_msg = ''
    
    curtemplate=runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ""
    tmp_vars["checkresult2"] = "failed"
    tmp_vars["checkvalue2"] = ""

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
    var_nginx_cfg_file = runcontext.get_extra_var('NGINX_CONFIG_FILES')
    base_path = os.path.split(var_nginx_cfg_file)[0]
    
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code,ret_msg
    
    try:
        errorServers = []
        checkServers = []
        errorPermission = []
        checkPermission = []
        errorNotExist = []
        errorMsg = []
        #get all server blocks
        serverblocks = find_in_directives(cfg_model,['http', 'server'])
        runcontext.log_debug("serverblocks: %s" % str(serverblocks))
        
        for server in serverblocks:
            server_name = ' '
            ssl_enable = 'off'
            ssl_certificate = None
            ssl_certificate_key = None
            for sub in server.value:
                key = sub.key
                value = sub.value
                if key == 'ssl':
                    ssl_enable = value
                elif key == 'ssl_certificate':
                    ssl_certificate = value
                elif key == 'ssl_certificate_key':
                    ssl_certificate_key = value
                elif key == 'server_name':
                    server_name = value
                #adapt to listen 443 ssl;
                elif key == 'listen':
                    if 'ssl' in value:
                        ssl_enable = 'on'
                else:
                    pass
            
            #check the configure
            if ssl_enable.lower() != 'on':
                errorServers.append(server_name)
            else:
                checkServers.append("server {\n…\nserver_name %s\n…\nssl on;\n…\n}" % server_name)
            
            if ssl_certificate is not None:
                if not os.path.isabs(ssl_certificate):
                    ssl_certificate = os.path.join(base_path,ssl_certificate)
                if not os.path.exists(ssl_certificate):
                    errorNotExist.append('File %s is not exists' % ssl_certificate)
                else:                    
                    st = os.stat(ssl_certificate)
                    st_mode = st.st_mode
                    owner_w = bool(st_mode & stat.S_IWUSR)
                    owner_x = bool(st_mode & stat.S_IXUSR)
                    group_r = bool(st_mode & stat.S_IRGRP)
                    group_w = bool(st_mode & stat.S_IWGRP)
                    group_x = bool(st_mode & stat.S_IXGRP)
                    other_r = bool(st_mode & stat.S_IROTH)
                    other_w = bool(st_mode & stat.S_IWOTH)
                    other_x = bool(st_mode & stat.S_IXOTH)
                    
                    if get_user_name(st.st_uid) != runuser or get_group_name(st.st_gid) != rungroup \
                       or group_r == True or group_w == True or group_x == True \
                       or other_r == True or other_w == True or other_x == True \
                       or owner_w == True or owner_x == True:
                        errorPermission.append('%s: %s %s:%s %s' % (server_name,ssl_certificate,get_user_name(st.st_uid),get_group_name(st.st_gid),("%04o" % stat.S_IMODE(st_mode))))
                    else:
                        checkPermission.append('%s: %s %s:%s %s' % (server_name,ssl_certificate,get_user_name(st.st_uid),get_group_name(st.st_gid),("%04o" % stat.S_IMODE(st_mode))))
            else:
                errorMsg.append("ssl_certificate is not exists")
                
            if ssl_certificate_key is not None:
                if not os.path.isabs(ssl_certificate_key):
                    ssl_certificate_key = os.path.join(base_path,ssl_certificate_key)
                if not os.path.exists(ssl_certificate_key):
                    errorNotExist.append('File %s is not exists' % ssl_certificate_key)
                else:                    
                    st = os.stat(ssl_certificate_key)
                    st_mode = st.st_mode
                    owner_w = bool(st_mode & stat.S_IWUSR)
                    owner_x = bool(st_mode & stat.S_IXUSR)
                    group_r = bool(st_mode & stat.S_IRGRP)
                    group_w = bool(st_mode & stat.S_IWGRP)
                    group_x = bool(st_mode & stat.S_IXGRP)
                    other_r = bool(st_mode & stat.S_IROTH)
                    other_w = bool(st_mode & stat.S_IWOTH)
                    other_x = bool(st_mode & stat.S_IXOTH)
                    
                    if get_user_name(st.st_uid) != runuser or get_group_name(st.st_gid) != rungroup \
                       or group_r == True or group_w == True or group_x == True \
                       or other_r == True or other_w == True or other_x == True \
                       or owner_w == True or owner_x == True:
                        errorPermission.append('%s: %s %s:%s %s' % (server_name,ssl_certificate_key,get_user_name(st.st_uid),get_group_name(st.st_gid),("%04o" % stat.S_IMODE(st_mode))))
                    else:
                        checkPermission.append('%s: %s %s:%s %s' % (server_name,ssl_certificate_key,get_user_name(st.st_uid),get_group_name(st.st_gid),("%04o" % stat.S_IMODE(st_mode))))
            else:
                errorMsg.append("ssl_certificate_key is not exists")

        if len(errorServers) > 0:
            tmp_vars["checkvalue1"] = 'The following servers did not enable ssl: %s' % ','.join(errorServers)
        else:
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "\n".join(checkServers)


        if len(errorPermission) > 0:
            tmp_vars["checkvalue2"] = '\n'.join(errorPermission)
        elif len(errorNotExist) > 0:
            tmp_vars["checkvalue2"] = tmp_vars["checkvalue2"] + '\n'.join(errorNotExist)
        elif len(errorMsg) > 0:
            tmp_vars["checkvalue2"] = tmp_vars["checkvalue2"] + '\n'.join(errorMsg)
        else:
            tmp_vars["checkresult2"] = "pass"
            tmp_vars["checkvalue2"] = "\n".join(checkPermission)

        if tmp_vars["checkresult1"] == 'pass' and tmp_vars["checkresult2"] == 'pass':
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
        else:
            ret_code = -1
        
        rc,out = curtemplate.render(runcontext.get_template_file('t_2_7_1.j2'),**tmp_vars)
        
        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out
            
    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code,ret_msg
