# -*- coding: UTF-8 -*-
import traceback

from linux.nginxparser import find_in_directives


def get_tags():
    return ['2', '2.7', '2.7.2']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all server nodes
        serverblocks = find_in_directives(cfg_model, ['http', 'server'])
        runcontext.log_debug("serverblocks: %s" % str(serverblocks))
        servernames = []
        vservers = []
        rservers = []
        vservers.append("server {")
        rservers.append("server {")
        for serverblock in serverblocks:
            servername = ' '
            checkflag = False
            # check config ssl_session_timeout  5m;
            svalues = []
            rvalues = []
            for sub in serverblock.value:
                key = sub.key
                if key.strip() == "#":
                    continue
                if key == 'ssl_session_timeout':
                    checkflag = True
                    rvalues.append("%s %s" % (key, sub.value))
                elif key == 'server_name':
                    servername = sub.value
                    svalues.append("server_name %s" % servername)
                    rvalues.append("server_name %s" % servername)

            if checkflag == False:
                servernames.append(servername)
                vservers.extend(svalues)
                vservers.append("}")
            else:
                rservers.extend(rvalues)
                rservers.append("}")

        if len(servernames) > 0:
            ret_code = -1
            tmp_vars["checkvalue1"] = "\n".join(vservers)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "\n".join(rservers)

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_7_2.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
