# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_in_directives


def get_tags():
    return ['2', '2.7', '2.7.3']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all server nodes
        serverblocks = find_in_directives(cfg_model, ['http', 'server'])
        runcontext.log_debug("serverblocks: %s" % str(serverblocks))

        checkresult = False
        j = 0  # block计数
        k = 0  # 当前block中匹配则计数加1
        nservers = []
        rservers = []
        svalues = []
        rvalues = []
        nservers.append("server {")
        rservers.append("server {")
        for serverblock in serverblocks:
            checkflag = False
            j = j + 1
            # check config ssl_protocols  TLSv1 TLSv1.1 TLSv1.2;
            i = 0  # i用于记录serverblock中ssl_protocols的配置次数

            svalues = []
            rvalues = []
            for sub in serverblock.value:
                key = sub.key
                if key.strip() == "#":
                    continue
                if key == 'ssl_protocols':
                    value = sub.value
                    i = i + 1
                    match_found = re.match(r'.*TLSv1.2', value)
                    if 1 == i:
                        if None != match_found:
                            checkflag = True
                            rvalues.append("%s %s" % (key, value))
                        else:
                            svalues.append("%s %s" % (key, value))

                    else:
                        if None == match_found:
                            checkflag = False
                            svalues.append("%s %s" % (key, value))
                        else:
                            checkflag = True
                elif "server_name" == key:
                    servername = sub.value
                    svalues.append("server_name %s" % servername)
                    rvalues.append("server_name %s" % servername)

            if checkflag == True:
                checkresult = True
                svalues = []
                rservers.extend(rvalues)
                k = k + 1
            nservers.extend(svalues)
        rservers.append("}")
        nservers.append("}")
        if j == k:
            checkresult = True
            nservers = []
        else:
            checkresult = False

        if checkresult == False:
            ret_code = -1
            tmp_vars["checkvalue1"] = "\n".join(nservers)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "\n".join(rservers)

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_7_3.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
