# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_in_directives

def get_tags():
    return ['2','2.7','2.7.6']
    
def run_task(runcontext):
    rc,out=run_audit(runcontext)
    return rc,out
    
def run_audit(runcontext):
    
    ret_code = 0
    ret_msg = ''
    
    curtemplate=runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''
        
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
        
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code,ret_msg
    
    try:
        #get all server nodes
        serverblocks = find_in_directives(cfg_model,['http','server'])
        runcontext.log_debug("serverblocks: %s" % str(serverblocks))
        servernames = []
        nservers = []
        rservers = []
        svalues = []
        rvalues = []
        key = ""
        value = ""
        for serverblock in serverblocks:
            match_found1 = None
            match_found2 = None
            #check config ssl_session_cache and ssl_session_timeout;
            server_name = " "
            nservers.append("server {")
            rservers.append("server {")
            svalues = []
            rvalues = []
            for sub in serverblock.value:
                key = sub.key
                value = sub.value
                
                if key == 'ssl_session_cache':
                    match_found1 = re.match(r'.+',value)
                    if match_found1 != None:
                        rvalues.append("%s %s"%(key, value))
                elif key == 'ssl_session_timeout':
                    match_found2 = re.match(r'.+',value)
                    if match_found2 != None:
                        rvalues.append("%s %s"%(key, value))
                elif key == 'server_name':
                    server_name = sub.value
                else:
                    pass

            if None == match_found1 or None == match_found2:
                servernames.append(server_name)
                nservers.append("server_name %s"%(server_name))
                rservers.append("server_name %s"%(server_name))
            else:
                svalues = []
                rservers.extend(rvalues)
                rservers.append("}")
                
            nservers.extend(svalues)
        nservers.append("}")

        if len(servernames) > 0:
            ret_code = -1
            tmp_vars["checkvalue1"] = "\n".join(nservers)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "\n".join(rservers)
            
        rc,out = curtemplate.render(runcontext.get_template_file('t_2_7_6.j2'),**tmp_vars)
        
        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out
            
    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code,ret_msg
