﻿# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_all_in_directives


def get_tags():
    return ['2', '2.9', '2.9.1']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkresult2"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all access_log and log_format nodes
        limit_zone = find_all_in_directives(cfg_model, 'limit_conn_zone')
        conn_zone = find_all_in_directives(cfg_model, 'limit_conn')
        runcontext.log_debug("limit_conn_zone: %s" % str(limit_zone))

        match_found1 = None

        match_found1_tmp = None

        evalues1 = []
        rvalues1 = []
        evalues2 = []
        rvalues2 = []
        check_limit = False
        check_zone = False
        zone_name = []



        if 0 < len(limit_zone):
            check_limit = True
            zone_name = []

            for sub in limit_zone:
                match_found1 = re.match(r'.+\szone=(.+):\d+', sub.value)
                if None == match_found1:
                    # fail
                    check_limit = False
                    evalues1.append("limit_zone %s" % sub.value)
                else:
                    rvalues1.append("limit_zone %s" % sub.value)
                    zone_name.append(match_found1.group(1))
        else:
            check_limit = False

        if  0 < len(limit_zone):
            check_zone = True
            for sub in conn_zone:
                re_match = "(.+)[: ]\\d+"
                match_found2 = re.match(re_match, sub.value)
                if None == match_found2:
                    # fail
                    check_zone = False
                    evalues2.append("limit_conn %s" % sub.value)
                else:
                    rvalues2.append("limit_conn %s" % sub.value)
                    zone = match_found2.group(1)
                    if zone not in zone_name:
                        check_zone = False
        else:
            check_zone = False


        if check_limit == False:
            tmp_vars["checkvalue1"] = "\\n".join(evalues1)
        else:
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "\\n".join(rvalues1)

        if check_zone == False:
            tmp_vars["checkvalue2"] = "\\n".join(evalues2)
        else:
            tmp_vars["checkresult2"] = "pass"
            tmp_vars["checkvalue2"] = "\\n".join(rvalues2)

        if tmp_vars["checkresult1"] == "pass" and tmp_vars["checkresult2"] == "pass":
            tmp_vars["policyresult"] = "pass"
        else:
            ret_code = -1

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_9_1.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
