﻿# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_all_in_directives


def get_tags():
    return ['2', '2.9', '2.9.2']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all access_log and log_format nodes
        limit_zone = find_all_in_directives(cfg_model, 'limit_rate')
        runcontext.log_debug("limit_rate: %s" % str(limit_zone))

        match_found1 = None
        match_found1_tmp = None

        evalues = []
        rvalues = []
        checkresult = False

        if 0 < len(limit_zone):
            checkresult = True
            for sub in limit_zone:
                match_found1 = re.match(r'.+', sub.value)
                if None == match_found1:
                    # fail
                    checkresult = False
                    evalues.append("limit_rate %s" % sub.value)
                else:
                    rvalues.append("limit_rate %s" % sub.value)

            match_found1 = match_found1_tmp


            if None != match_found1:
                checkresult = False

        else:
            checkresult = False

        if checkresult == False:
            ret_code = -1
            tmp_vars["checkvalue1"] = "\\n".join(evalues)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "\\n".join(rvalues)

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_9_2.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
