﻿# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_in_directives

def get_tags():
    return ['1','1.1','1.1.1']

def run_task(runcontext):
    rc,out=run_audit(runcontext)
    return rc,out

def run_audit(runcontext):

    ret_code = 0
    ret_msg = ''

    curtemplate=runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code,ret_msg

    try:
        #get all location nodes
        errMsg = ''
        checkMsg = []
        location_count = 0
        locationblocks = find_in_directives(cfg_model,['http','server','location'])
        msg_infos = ['svn', 'git', 'sql', 'bak', 'old', 'tar', 'gz', 'tgz', 'zip', '7z', 'rar', 'DS_store']
        msg_infos_str = ['csv', 'inc', 'config', 'conf', 'sh', 'sw[op]', 'bash_rc', 'tmp', 'ht', 'log', 'db', 'entries']

        if 0 == len(locationblocks):
            errMsg += 'No location directive was found.'
        else:
            for locationblock in locationblocks:
                if 2 == len(locationblock.args) and '~*' == locationblock.args[0]:
                    test_str = locationblock.args[1]
                    test_str = test_str.lstrip('\s').lstrip('(').rstrip(')')
                    testList = re.split(r'\(|\)|\|',test_str)
                    reg = '\.' == testList[0] and '$' == testList[-1] and ('svn' in testList[1:-1] or 'git' in  testList[1:-1] or 'sql' in  testList[1:-1] or 'bak' in  testList[1:-1] or 'old' in testList[1:-1] or 'tar' in  testList[1:-1] or 'gz' in  testList[1:-1] or 'zip' in  testList[1:-1] or '7z' in  testList[1:-1] or 'rar' in  testList[1:-1] or 'DS_store' in  testList[1:-1])
                    res = '\.' == testList[0] and '$' == testList[-1] and ('csv' in testList[1:-1] or 'inc' in testList[1:-1] or 'config' in testList[1:-1] or 'conf' in testList[1:-1] or 'sh' in testList[1:-1] or 'sw[op]' in testList[1:-1] or 'bash_rc' in  testList[1:-1] or 'tmp' in  testList[1:-1] or 'ht' in  testList[1:-1] or 'log' in  testList[1:-1] or 'db' in  testList[1:-1] or 'entries' in  testList[1:-1])
                    key = [svalue.key for svalue in locationblock.value]
                    value = [svalue.value for svalue in locationblock.value]
                    if reg:
                        location_count += 1
                        if 'deny' in key and 'all' in value:
                            checkMsg.append("location %s {\n  ...  \ndeny  all;\n  ...  \n}\n" % (" ".join(locationblock.args)))
                            if 'failed' in errMsg:
                                errMsg = errMsg.replace('failed','')
                            for msg_info in msg_infos:
                                if msg_info not in testList[1:-1]:
                                    errMsg = errMsg + msg_info + ' '
                        else:
                            errMsg += 'deny all not set in %s. ' % locationblock.args
                    elif res:
                        location_count += 1
                        if 'deny' in key and 'all' in value:
                            checkMsg.append("location %s {\n  ...  \ndeny  all;\n  ...  \n}\n" % (" ".join(locationblock.args)))
                            if 'failed' in errMsg:
                                errMsg.replace('failed','')
                            for msg_info in msg_infos_str:
                                if msg_info not in testList[1:-1]:
                                    errMsg = errMsg + msg_info + ' '
                        else:
                            errMsg += 'deny all not set in %s. ' % locationblock.args
                elif 2 < len(locationblock.args) and '~*' == locationblock.args[0]:
                    errMsg += 'Location setting error in %s.' % locationblock.args
                    test_str = locationblock.args[1]
                    test_str = test_str.lstrip('\s').lstrip('(').rstrip(')')
                    testList = re.split(r'\(|\)|\|',test_str)
                    reg = '\.' == testList[0] and '$' == testList[-1] and ('svn' in testList[1:-1] or 'git' in  testList[1:-1] or 'sql' in  testList[1:-1] or 'bak' in  testList[1:-1] or 'old' in testList[1:-1] or 'tar' in  testList[1:-1] or 'gz' in  testList[1:-1] or 'zip' in  testList[1:-1] or '7z' in  testList[1:-1] or 'rar' in  testList[1:-1] or 'DS_store' in  testList[1:-1])
                    res = '\.' == testList[0] and '$' == testList[-1] and ('csv' in testList[1:-1] or 'inc' in testList[1:-1] or 'config' in testList[1:-1] or 'conf' in testList[1:-1] or 'sh' in testList[1:-1] or 'sw[op]' in testList[1:-1] or 'bash_rc' in  testList[1:-1] or 'tmp' in  testList[1:-1] or 'ht' in  testList[1:-1] or 'log' in  testList[1:-1] or 'db' in  testList[1:-1] or 'entries' in  testList[1:-1])
                    if reg or res:
                        location_count += 1

        if location_count == 0:
            errMsg = "No location exists for this configuration."

        if 0 == len(errMsg):
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "".join(checkMsg)
        else:
            tmp_vars["checkvalue1"] = errMsg
            ret_code = -1

        rc,out = curtemplate.render(runcontext.get_template_file('a_1_1_1.j2'),**tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code,ret_msg
