﻿# -*- coding: UTF-8 -*-
import json
import os
import re


def get_tags():
    return ['2', '2.2', '2.2.2']


def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params, "chdir", None)
    runcontext.set_mod_params_instance(ansible_params, "executable", None)
    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansible_params, module)
    return rc, out


def run_audit(runcontext, ansible_params, module):
    curtemplate = runcontext.get_template()
    nginxbin = runcontext.get_extra_var('NGINX_BIN')
    allcfgs = runcontext.get_extra_var('ALL_NGINX_CONFIG_FILES')

    tmpvalue = []
    tmp_vars = {}

    tmp_vars["policyresult"] = "failed"
    tmp_vars["systemvalue"] = ''
    tmp_vars["scancommand1"] = "%s/nginx -V" % runcontext.replace_inject_characters(nginxbin)
    tmp_vars["systemvalue1"] = ""

    ret_code = -2

    if os.path.exists(nginxbin) == False:
        tmp_vars["systemvalue"] = 'cannot access %s: No such file or directory.' % nginxbin

    else:
        runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand1"])
        arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansible_params)
        runcontext.log_debug_invoke_result(arc1, astdout1, astderr1)
        runcontext.sleep()

        if arc1 != 0:
            return (-2, "Failed to invoke module: %s." % astdout1)

        resultjson1 = json.loads(astdout1)
        astdout1 = resultjson1["stdout"]
        astderr1 = resultjson1["stderr"]

        match1_found1 = re.match(".*--with-http_dav_module", astderr1)
        # if not found--> pass,else check following
        if match1_found1 == None:
            tmp_vars["systemvalue"] = "Compile parameters do not include --with-http_dav_module"
            tmp_vars["policyresult"] = "pass"
        else:

            allcfgs1 = []
            for c in allcfgs:
                allcfgs1.append(runcontext.replace_inject_characters(c))

            tmp_vars["scancommand2"] = "cat %s | grep -v \"^\s*#\" | egrep '(dav_methods|dav_ext_methods|create_full_put_path|dav_access|auth_basic|auth_basic_user_file)'" % " ".join(allcfgs1)

            if len(allcfgs1) != 0:
                runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand2"])
                arc2, astdout2, astderr2 = runcontext.invoke_module(module, ansible_params)
                runcontext.log_debug_invoke_result(arc2, astdout2, astderr2)
                runcontext.sleep()
            else:
                arc2 = 0
                astdout2 = ''
                astderr2 = ''

            if arc2 != 0:
                return (-2, "Failed to invoke module: %s." % astdout2)
            resultjson2 = json.loads(astdout2)
            astdout2 = resultjson2["stdout"]
            astderr2 = resultjson2["stderr"]

            match2_found = re.match(".*dav_method ", astdout2)
            match2_found1 = re.match(".*dav_ext_methods", astdout2)
            match2_found2 = re.match(".*create_full_put_path", astdout2)
            match2_found3 = re.match(".*dav_access", astdout2)
            match2_found4 = re.match(".*auth_basic_user_file", astdout2)
            match2_found5 = re.match(".*auth_basic", astdout2)

            if match2_found != None:
                tmpvalue.append("dav_method")
            if match2_found1 != None:
                tmpvalue.append("dav_ext_methods")
            if match2_found2 != None:
                tmpvalue.append("create_full_put_path")
            if match2_found3 != None:
                tmpvalue.append("dav_access")
            if match2_found4 != None:
                tmpvalue.append("auth_basic_user_file")
            if match2_found5 != None:
                tmpvalue.append("auth_basic")

            if len(''.join(tmpvalue)) == 6:
                tmp_vars["policyresult"] = "pass"
            else:
                tmp_vars["policyresult"] = "failed"
                tmp_vars["systemvalue"] = astdout2

    if tmp_vars["policyresult"] == "pass":
        ret_code = 0
    else:
        ret_code = -1

    rc, out = curtemplate.render(runcontext.get_template_file('t_2_2_2.j2'), **tmp_vars)
    if rc == False:
        return (-2, "Failed to render template.%s" % out)
    return ret_code, out
