﻿# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_in_directives


def get_tags():
    return ['2', '2.2', '2.2.6']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all http nodes
        serverblocks = find_in_directives(cfg_model, ['http', 'server'])
        runcontext.log_debug("serverblocks: %s" % str(serverblocks))
        errMsg = []
        retCheckMsg = []
        for serverblock in serverblocks:
            for sub in serverblock.value:
                key = sub.key
                if key == 'listen':
                    value = sub.value

                    if re.match(r"\s*0\.0\.0\.0", value) != None:
                        errMsg.append(value)
                    elif re.match(r"\d+\s*$", value) != None:
                        errMsg.append(value)
                    elif re.match(r"\*.*\d+\s*$", value) != None:
                        errMsg.append(value)
                    elif re.match(r"\[::\].*\d+\s*$", value) != None:
                        errMsg.append(value)
                    else:
                        retCheckMsg.append(value)

        if len(errMsg) > 0:
            ret_code = -1
            tmp_vars["checkvalue1"] = "listen: %s" % ','.join(errMsg)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "listen: %s" % ','.join(retCheckMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_2_6.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
