﻿# -*- coding: UTF-8 -*-
import traceback

from linux.nginxparser import find_in_directives


def get_tags():
    return ['2', '2.2', '2.2.9']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all http nodes
        httpblocks = find_in_directives(cfg_model, ['http'])
        runcontext.log_debug("httpblocks: %s" % str(httpblocks))
        checkresult = True
        cur_cgs = []

        for httpblock in httpblocks:
            flag1 = False
            flag2 = False
            flag3 = False
            flag4 = False
            for sub in httpblock.value:
                key = sub.key
                values = sub.value
                if key == "client_body_timeout":
                    cur_cgs.append('client_body_timeout %s' % str(values))
                    flag1 = True

                elif key == 'client_header_timeout':
                    cur_cgs.append('client_header_timeout %s' % str(values))
                    flag2 = True

                elif key == "keepalive_timeout":
                    cur_cgs.append('keepalive_timeout %s' % str(values))
                    flag3 = True

                elif key == "send_timeout":
                    cur_cgs.append('send_timeout %s' % str(values))
                    flag4 = True

            reg = not flag1 or not flag2 or not flag3 or not flag4
            if reg:
                checkresult = False
                break

        if checkresult == False:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\\n'.join(cur_cgs)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(cur_cgs)

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_2_9.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
