﻿# -*- coding: UTF-8 -*-
import os
import re
import json
import copy

from linux.nginxparser import find_all_in_directives

def get_tags():
    return ['2','2.3','2.3.1']

def run_task(runcontext):
    ansibleParams=runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansibleParams,"chdir",None)
    runcontext.set_mod_params_instance(ansibleParams,"executable",None)
    module=runcontext.get_module_file("shell.py")
    rc,out=run_audit(runcontext,ansibleParams,module)
    return rc,out

def run_audit(runcontext,ansibleParams,module):
    curtemplate=runcontext.get_template()
    nginxbin = runcontext.get_extra_var('NGINX_BIN')
    allcfg1 = runcontext.get_extra_var('ALL_NGINX_CONFIG_FILES')
    nginx_pid = runcontext.get_extra_var('NGINX_PID')
    nginx_pid = runcontext.replace_inject_characters(nginx_pid)
    allcfgs = copy.deepcopy(allcfg1)
    allcfgs.append(nginxbin)

    retCode = 0
    tmpVars = {}
    tmpVars["policyresult"] = "failed"
    tmpVars["systemvalue1"] = ""
    tmpPath = []
    errPath = []
    index = 0

    nginx_user = '''ps -uwp %s | grep nginx | awk -F' ' '{print $1}' ''' % nginx_pid
    runcontext.set_mod_params_instance(ansibleParams, "command", nginx_user)
    arc3, astdout3, astderr3 = runcontext.invoke_module(module, ansibleParams)
    runcontext.sleep()
    if arc3 != 0:
        return -2, "Failed to invoke module: %s." % astdout3
    resultjson3 = json.loads(astdout3)
    astdout3 = resultjson3["stdout"]
    runcontext.log_debug_json(resultjson3)
    match3_found = re.match("^\s*root\s*$", astdout3)

    nginx_group = '''GID=$(cat /etc/passwd | grep "^\s*%s:" | awk -F':' '{print $4}');cat /etc/group | grep ":$GID:" | awk -F':' '{print $1}' ''' % astdout3
    runcontext.set_mod_params_instance(ansibleParams, "command", nginx_group)
    arc4, astdout4, astderr4 = runcontext.invoke_module(module, ansibleParams)
    runcontext.sleep()
    if arc4 != 0:
        return -2, "Failed to invoke module: %s." % astdout4
    resultjson4 = json.loads(astdout4)
    astdout4 = resultjson4["stdout"]
    runcontext.log_debug_json(resultjson4)

    if match3_found:
        cfg_model = runcontext.get_parameter('_nginx_cfg_model')
        tmpusers = find_all_in_directives(cfg_model, 'user')
        runcontext.log_debug("tmpusers: %s" % str(tmpusers))
        if len(tmpusers) == 0:
            runuser = 'nobody'
            rungroup = 'nobody'
        else:
            values = runcontext.replace_inject_characters(tmpusers[-1].value)
            values_list = values.split()
            if len(values_list) == 1:
                runuser = values
                rungroup = values
            else:
                runuser = values_list[0]
                rungroup = values_list[-1]
    else:
        runuser = astdout3
        rungroup = astdout4

    for path in allcfgs:
        tmpPath = []
        sepPaths = path.rsplit("%s"%(os.sep))
        sepPaths[0] = os.sep
        index = 0
        for spath in sepPaths:
            if spath == "":
                continue

            if "conf" in tmpPath:
                continue
            tmpPath.append(spath)
            filedir = runcontext.replace_inject_characters("%s"%(os.sep).join(tmpPath))
            if len(tmpPath) > 1:
                filedir = filedir.replace(os.sep+os.sep,os.sep)

            tmpVars["checkresult1"] = "failed"
            tmpVars["scancommand"] = r'find %s -maxdepth 0 \( \( ! -user root -a ! -user %s \) -o \( ! -group root -a  ! -group %s \)   -o -perm /g=w,o=w  \) ' % (filedir,runuser,rungroup)
            tmpVars["scancommand"] = tmpVars["scancommand"] + r'''-exec stat -L --print "%n %U:%G %a;" {} \; 2>/dev/null'''
            index = index + 1
            runcontext.set_mod_params_instance(ansibleParams,"command",tmpVars["scancommand"])
            arc1,astdout1,astderr1=runcontext.invoke_module(module,ansibleParams)
            runcontext.sleep()
            if arc1 != 0:
                return (-2, "Failed to invoke module: %s." % astdout1)

            resultjson1 = json.loads(astdout1)
            astdout1 = resultjson1["stdout"]
            astderr1 = resultjson1["stderr"]

            runcontext.log_debug_json(resultjson1)
            match1_found=re.match("^$",astdout1)
            if match1_found == None:
                retCode = -1
                if astdout1 not in errPath:
                    errPath.append(astdout1)

    if len(errPath) != 0:
        tmpVars["systemvalue1"] = "\\n".join(errPath)
    else:
        tmpVars["systemvalue1"] = "Meet permission requirements"
        tmpVars["policyresult"]="pass"

    rc,out=curtemplate.render(runcontext.get_template_file('t_2_3_1.j2'),**tmpVars)

    if rc == False:
        return (-2,"Failed to render template.%s" % out)

    return retCode,out
