﻿# -*- coding: UTF-8 -*-
import os
import re
import json
import traceback

from linux.nginxparser import find_all_in_directives

def get_tags():
    return ['2', '2.3', '2.3.4']
    
def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params, "chdir", None)
    runcontext.set_mod_params_instance(ansible_params, "executable", None)
    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansible_params, module)
    return rc, out
    
def run_audit(runcontext, ansible_params, module):
    nginx_pid = runcontext.get_extra_var('NGINX_PID')
    nginx_pid = runcontext.replace_inject_characters(nginx_pid)
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    nginx_user = '''ps -uwp %s | grep nginx | awk -F' ' '{print $1}' ''' % nginx_pid
    runcontext.set_mod_params_instance(ansible_params, "command", nginx_user)
    arc3, astdout3, astderr3 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    if arc3 != 0:
        return -2, "Failed to invoke module: %s." % astdout3
    resultjson3 = json.loads(astdout3)
    astdout3 = resultjson3["stdout"]
    runcontext.log_debug_json(resultjson3)
    match3_found = re.match("^\s*root\s*$", astdout3)

    nginx_group = '''GID=$(cat /etc/passwd | grep "^\s*%s:" | awk -F':' '{print $4}');cat /etc/group | grep ":$GID:" | awk -F':' '{print $1}' ''' % astdout3
    runcontext.set_mod_params_instance(ansible_params, "command", nginx_group)
    arc4, astdout4, astderr4 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    if arc4 != 0:
        return -2, "Failed to invoke module: %s." % astdout4
    resultjson4 = json.loads(astdout4)
    astdout4 = resultjson4["stdout"]
    runcontext.log_debug_json(resultjson4)

    if match3_found:
        tmpusers = find_all_in_directives(cfg_model, 'user')
        runcontext.log_debug("tmpusers: %s" % str(tmpusers))
        if len(tmpusers) == 0:
            runuser = 'nobody'
            rungroup = 'nobody'
        else:
            values = runcontext.replace_inject_characters(tmpusers[-1].value)
            values_list = values.split()
            if len(values_list) == 1:
                runuser = values
                rungroup = values
            else:
                runuser = values_list[0]
                rungroup = values_list[-1]
    else:
        runuser = astdout3
        rungroup = astdout4

    ret_code = 0
    ret_msg = ''
    
    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''
    tmp_vars["checkresult2"] = "failed"
    tmp_vars["checkvalue2"] = ''
    
    cfgPath = runcontext.get_extra_var('NGINX_CONFIG_FILES')

    base_cfg = os.path.split(cfgPath)[0]
    m = re.search("conf",base_cfg)
    if m != None:
        base_cfg = base_cfg[0:m.start()].strip().rstrip(os.sep)
    
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg
    
    try:
        # get all root nodes
        rootblocks = find_all_in_directives(cfg_model, 'root')       
        errmsg1 = []
        cmds1 = []
        
        for block in rootblocks:
            runcontext.log_debug(block)
            value = block.value.strip()
            values = value.split()
            root_file = ''
            if len(values) == 0:
                runcontext.log_debug('root configure value is incorrect : %s.' % value)
                continue
            
            root_file = values[0]
            if os.path.isabs(root_file) is False:
                # 相对路径，一般为标准部署
                root_file = os.path.join(base_cfg,root_file)
            
            thiscmd = '''stat -L --format="%n %U:%G %a" '''+root_file + ''' 2>/dev/null'''
            cmds1.append(thiscmd)
            runcontext.set_mod_params_instance(ansible_params, "command", thiscmd)
            arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansible_params)
            runcontext.log_debug_invoke_result(arc1,astdout1,astderr1)
            runcontext.sleep()
            if arc1 != 0:
                return (-2, "Failed to invoke module: %s." % astdout1)

            resultjson1 = json.loads(astdout1)
            astdout1 = resultjson1["stdout"]
            astderr1 = resultjson1["stderr"]
            m = re.match(r"(.*) (?P<user>\w+)(:)(?P<group>\w+) (?P<mod>\d00)", astdout1)
            if m == None:
                if astdout1 != "":
                    errmsg1.append(astdout1)
            else:
                if m.group('user') not in ['root',runuser] or m.group('group') not in ['root',rungroup]:
                    errmsg1.append(astdout1)
                    continue
        
        tmp_vars["scancommand1"] = ';'.join(cmds1)
        if len(errmsg1) >0 :
            tmp_vars["checkvalue1"] = '\n'.join(errmsg1)
        else:
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = ""
            
            
        if tmp_vars["checkresult1"] == 'pass':
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
        else:
            ret_code = -1            
        
        rc, out = curtemplate.render(runcontext.get_template_file('t_2_3_4.j2'), **tmp_vars)
        
        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out
            
    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
