﻿# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_all_in_directives


def get_tags():
    return ['2', '2.5', '2.5.1']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    cfg_model = runcontext.get_parameter('_nginx_cfg_model')

    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code, ret_msg

    try:
        # get all access_log and log_format nodes
        access_lognodes = find_all_in_directives(cfg_model, 'access_log')
        log_formatnodes = find_all_in_directives(cfg_model, 'log_format')
        runcontext.log_debug("access_lognodes: %s" % str(access_lognodes))
        runcontext.log_debug("log_formatnodes: %s" % str(log_formatnodes))
        match_found1 = None
        match_found1_tmp = None
        match_found2 = None
        match_found2_tmp = 'success'

        evalues = []
        rvalues = []
        checkresult = False

        if 0 < len(access_lognodes) and len(log_formatnodes) == 0:
            checkresult = True
            for sub in access_lognodes:
                match_found1 = re.match(r'off', sub.value)
                if None != match_found1:
                    # fail
                    checkresult = False
                    evalues.append("access_log %s" % sub.value)
                else:
                    rvalues.append("access_log %s" % sub.value)
        elif 0 < len(access_lognodes) and 0 < len(log_formatnodes):
            checkresult = True
            # check if access_log on
            for sub in access_lognodes:
                match_found1 = re.match(r'off', sub.value)
                if None != match_found1:
                    # fail
                    match_found1_tmp = match_found1
                    evalues.append("access_log %s" % sub.value)
                else:
                    rvalues.append("access_log %s" % sub.value)

            # check log_format config

            for sub in log_formatnodes:
                log_format_values = sub.value
                log_format_values = log_format_values.replace('\n', '')
                match_found2 = re.match(r'.*\$remote_addr', log_format_values) and \
                               re.match(r'.*\$remote_user', log_format_values) and \
                               re.match(r'.*\$time_local', log_format_values) and \
                               re.match(r'.*\$request', log_format_values) and \
                               re.match(r'.*\$status', log_format_values)
                if None == match_found2:
                    match_found2_tmp = match_found2
                    evalues.append("log_format %s" % sub.value)
                else:
                    rvalues.append("log_format %s" % sub.value)

            match_found1 = match_found1_tmp
            match_found2 = match_found2_tmp

            if None != match_found1 or None == match_found2:
                checkresult = False

        else:
            checkresult = False

        if checkresult == False:
            ret_code = -1
            tmp_vars["checkvalue1"] = "\\n".join(evalues)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = "\\n".join(rvalues)

        rc, out = curtemplate.render(runcontext.get_template_file('t_2_5_1.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
