# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_in_directives

def get_tags():
    return ['2','2.7','2.7.4']
    
def run_task(runcontext):
    rc,out = run_audit(runcontext)
    return rc,out
    
def run_audit(runcontext):
    
    ret_code = 0
    ret_msg = ''
    
    curtemplate=runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''
        
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
        
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code,ret_msg
    
    try:
        #get all server nodes
        serverblocks = find_in_directives(cfg_model,['http', 'server'])
        runcontext.log_debug("serverblocks: %s" % str(serverblocks))

        svalues = []

        checkresult = False
        i = 0       #block计数
        j = 0       #当前block中匹配则计数加1
        for serverblock in serverblocks:
            checkflag = False
            i = i + 1
            match_found1 = False
            match_found2 = None
            #check config ssl_ciphers and ssl_prefer_server_ciphers;
            svalues.append("server {")
            sha_list = ['ECDHE-RSA-AES256-GCM-SHA384',
                        'ECDHE-RSA-AES128-GCM-SHA256',
                        'DHE-RSA-AES256-GCM-SHA384',
                        'DHE-RSA-AES128-GCM-SHA256', 'ECDHE-RSA-AES256-SHA384',
                        'ECDHE-RSA-AES256-SHA', 'DHE-RSA-AES256-SHA256',
                        'DHE-RSA-AES256-SHA', 'AES256-GCM-SHA384',
                        'AES128-GCM-SHA256', 'AES256-SHA256', 'AES256-SHA',
                        'HIGH', '!MEDIUM', '!LOW', '!aNULL', '!eNULL',
                        '!EXPORT', '!DES', '!MD5', '!PSK', '!RC4', '@STRENGTH']
            for sub in serverblock.value:
                key = sub.key
                value = sub.value
                if key == 'ssl_ciphers':
                    svalues.append("%s %s"%(key, value))
                    value = value.replace("'", "")
                    match_found1 = True
                    value_list = value.split(':')
                    for sha in value_list:
                        if sha not in sha_list:
                            match_found1 = False
                            break
                elif key == 'ssl_prefer_server_ciphers':
                    svalues.append("%s %s"%(key, value))
                    match_found2 = re.match(r'\s*on\s*',value)
                elif key == "server_name":
                    svalues.append("%s %s"%(key, value))
                else:
                    pass

            svalues.append("}")

            if match_found1 and None != match_found2:
                checkflag = True

                                
            if checkflag == True:
                checkresult = True
                j = j + 1
        
        if i == j:
            checkresult = True
        else:
            checkresult = False

        tmp_vars["checkvalue1"] = "\n".join(svalues)
        if checkresult == False:
            ret_code = -1
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"

            
        rc,out = curtemplate.render(runcontext.get_template_file('t_2_7_4.j2'),**tmp_vars)
        
        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out
            
    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code,ret_msg
