# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_all_in_directives

def get_tags():
    return ['2','2.7','2.7.8']
    
def run_task(runcontext):
    rc,out=run_audit(runcontext)
    return rc,out
    
def run_audit(runcontext):
    
    ret_code = 0
    ret_msg = ''
    
    curtemplate=runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''
        
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
        
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code,ret_msg
    
    try:
        #get all server nodes
        serverblocks = find_all_in_directives(cfg_model,['server'])
        runcontext.log_debug("serverblocks: %s" % str(serverblocks))
        tmpMsg = []
        totalMsg = []

        checkresult = False
        i = 0       #block计数
        j = 0       #当前block中匹配则计数加1
        for serverblock in serverblocks:
            checkflag = False
            i = i + 1
            match_found1 = None
            match_found2 = None
            match_found3 = None
            match_found4 = None
            tmpMsg.append('server {')
            #check config ssl_stapling,ssl_stapling_verify,ssl_trusted_certificate and resolver;
            for sub in serverblock.value:
                key = sub.key
                if key == 'server_name':
                    tmpMsg.append('    server_name   %s;' % (sub.value))
                if key == 'ssl_stapling':
                    value1 = sub.value
                    match_found1 = re.match(r'\s*on',value1)
                    tmpMsg.append('    ssl_stapling   %s;' % (value1))
                if key == 'ssl_stapling_verify':
                    value2 = sub.value
                    match_found2 = re.match(r'\s*on',value2)
                    tmpMsg.append('    ssl_stapling_verify   %s;' % (value2))
                if key == 'ssl_trusted_certificate':
                    value3 = sub.value
                    match_found3 = re.match(r'\s*conf\.d/cert/AlphaSSLroot\.crt',value3)
                    tmpMsg.append('    ssl_trusted_certificate   %s;' % (value3))
                if key == 'resolver':
                    value4 = sub.value
                    match_found4 = re.match(r'\s*\d+\.\d+\.\d+\.\d+',value4)
                    tmpMsg.append('    resolver   %s;' % (value4))

            if None != match_found1 and None != match_found2 and None != match_found3 and None != match_found4:
                checkflag = True

            tmpMsg.append('}')
            totalMsg.append('\n'.join(tmpMsg))
            tmpMsg = []
                                
            if checkflag == True:
                j = j + 1

        if i == j:
            checkresult = True
        else:
            checkresult = False      
        if checkresult == False:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(totalMsg)
        else:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(totalMsg)
            
        rc,out = curtemplate.render(runcontext.get_template_file('t_2_7_8.j2'),**tmp_vars)
        
        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out
            
    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code,ret_msg
