# -*- coding: UTF-8 -*-
import re
import traceback

from linux.nginxparser import find_all_in_directives
from linux.nginxparser import find_in_directives

def get_tags():
    return ['2','2.8','2.8.1']
    
def run_task(runcontext):
    rc,out=run_audit(runcontext)
    return rc,out
    
def run_audit(runcontext):
    
    ret_code = 0
    ret_msg = ''
    
    curtemplate=runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''
        
    cfg_model = runcontext.get_parameter('_nginx_cfg_model')
        
    if cfg_model is None:
        ret_code = -2
        ret_msg = 'Parse Nginx configure file failed.'
        return ret_code,ret_msg
    
    try:
        #get all http nodes
        httpblocks = find_all_in_directives(cfg_model,['http'])
        runcontext.log_debug("httpblocks: %s" % str(httpblocks))
        serverblocks = find_in_directives(cfg_model,['http', 'server'])
        match_found1 = None
        match_found2 = None
        hResultList = []
        sResultList = []
        checkFlagHttp = False
        checkFlagServer = False
        tmpHMsg = []
        tmpSMsg = []
        totalMsg = []
        server_flag = True

        if 0 == len(httpblocks):
            totalMsg.append("Not find any http block!")      
        else:
            for httpblock in httpblocks:
                tmpHMsg.append('http {')
                match_found1 = None
                # check in http
                for sub in httpblock.value:
                    key = sub.key
                    if isinstance(key,str):
                        if 'limit_req_zone' == key:
                            match_found1 = re.match(r'^\$binary_remote_addr\s+zone\s*=.+:\d+m\s+rate\s*=\s*\d+r/s$',sub.value.strip())
                            tmpHMsg.append('    limit_req_zone %s;' % (sub.value))
                            checkFlagHttp = False
                            if None != match_found1:
                                checkFlagHttp = True
                                
                    elif server_flag:
                        server_flag = False
                        for serverblock in serverblocks:
                            match_found2 = None
                            tmpSMsg.append('    server {')
                            # check in server
                            checkFlagServer = False
                            for sub2 in serverblock.value:
                                if 'server_name' == "".join(sub2.key):
                                    serName = sub2.value
                                    tmpSMsg.append('        server_name %s;' % serName)
                                elif 'limit_req' == "".join(sub2.key):
                                    match_found2 = re.match(r'^zone\s*=.+\s+burst\s*=\s*[1-3]',sub2.value.strip())
                                    tmpSMsg.append('        limit_req %s;' % (sub2.value))
                                    if None != match_found2:
                                        checkFlagServer = True
                            sResultList.append(checkFlagServer)
                            tmpSMsg.append('    }')
                            tmpHMsg.extend(tmpSMsg)
                            tmpSMsg = []
                            
                hResultList.append(checkFlagHttp)
                tmpHMsg.append('}')
                totalMsg.extend(tmpHMsg)
                tmpHMsg = []
    
        if 0 != len(hResultList) and 0 != len(sResultList):
            if False not in hResultList and False not in sResultList:
                tmp_vars["policyresult"] = "pass"
                tmp_vars["checkresult1"] = "pass"
                tmp_vars["checkvalue1"] = '\n'.join(totalMsg)
                ret_code = 0
            else:
                tmp_vars["checkvalue1"] = '\n'.join(totalMsg)
                ret_code = -1
        else:
            tmp_vars["checkvalue1"] = '\n'.join(totalMsg)
            ret_code = -1
        rc,out = curtemplate.render(runcontext.get_template_file('t_2_8_1.j2'),**tmp_vars)
        
        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out
            
    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code,ret_msg
